@@ -245,7 +245,7 @@ DamageExtCreate(DrawablePtr pDrawable, DamageReportLevel level,
         return NULL;
     }
 
-    if (!AddResource(id, DamageExtType, (pointer) pDamageExt))
+    if (!AddResource(id, DamageExtType, (void *) pDamageExt))
         return NULL;
 
     DamageExtRegister(pDrawable, pDamageExt->pDamage,
@@ -568,26 +568,8 @@ SProcDamageDispatch(ClientPtr client)
     return (*SProcDamageVector[stuff->damageReqType]) (client);
 }
 
-static void
-DamageClientCallback(CallbackListPtr *list, pointer closure, pointer data)
-{
-    NewClientInfoRec *clientinfo = (NewClientInfoRec *) data;
-    ClientPtr pClient = clientinfo->client;
-    DamageClientPtr pDamageClient = GetDamageClient(pClient);
-
-    pDamageClient->critical = 0;
-    pDamageClient->major_version = 0;
-    pDamageClient->minor_version = 0;
-}
-
- /*ARGSUSED*/ static void
-DamageResetProc(ExtensionEntry * extEntry)
-{
-    DeleteCallback(&ClientStateCallback, DamageClientCallback, 0);
-}
-
 static int
-FreeDamageExt(pointer value, XID did)
+FreeDamageExt(void *value, XID did)
 {
     DamageExtPtr pDamageExt = (DamageExtPtr) value;
 
@@ -757,13 +739,10 @@ DamageExtensionInit(void)
         (&DamageClientPrivateKeyRec, PRIVATE_CLIENT, sizeof(DamageClientRec)))
         return;
 
-    if (!AddCallback(&ClientStateCallback, DamageClientCallback, 0))
-        return;
-
     if ((extEntry = AddExtension(DAMAGE_NAME, XDamageNumberEvents,
                                  XDamageNumberErrors,
                                  ProcDamageDispatch, SProcDamageDispatch,
-                                 DamageResetProc, StandardMinorOpcode)) != 0) {
+                                 NULL, StandardMinorOpcode)) != 0) {
         DamageReqCode = (unsigned char) extEntry->base;
         DamageEventBase = extEntry->eventBase;
         EventSwapVector[DamageEventBase + XDamageNotify] =
